////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 1998 by Thierry Maurel
// All rights reserved
//
// Distribute freely, except: don't remove my name from the source or
// documentation (don't take credit for my work), mark your changes (don't
// get me blamed for your possible bugs), don't alter or remove this
// notice.
// No warrantee of any kind, express or implied, is included with this
// software; use at your own risk, responsibility for damages (if any) to
// anyone resulting from the use of this software rests entirely with the
// user.
//
// Send bug reports, bug fixes, enhancements, requests, flames, etc., and
// I'll try to keep a version up to date.  I can be reached as follows:
//    tmaurel@caramail.com   (or tmaurel@hol.fr)
//
////////////////////////////////////////////////////////////////////////////////
// File    : AccelMapPage.cpp
// Project : AccelsEditor
////////////////////////////////////////////////////////////////////////////////
// Version : 1.0                       * Author : T.Maurel
// Date    : 17.08.98
//
// Remarks : implementation file, cf CAccelMapDlg.cpp
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "AccelMapPage.h"

#include "CmdAccelOb.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// CAccelMapPage property page
/////////////////////////////////////////////////////////////////////////////
//
//
IMPLEMENT_DYNCREATE(CAccelMapPage, CPropertyPage)

/////////////////////////////////////////////////////////////////////////////
//
//
CAccelMapPage::CAccelMapPage(CAcceleratorManager* pAccelManager)
		: CPropertyPage(CAccelMapPage::IDD)
{
	//{{AFX_DATA_INIT(CAccelMapPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pAccelManager = pAccelManager;
}


/////////////////////////////////////////////////////////////////////////////
//
//
CAccelMapPage::~CAccelMapPage()
{
}


/////////////////////////////////////////////////////////////////////////////
//
//
void CAccelMapPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAccelMapPage)
	DDX_Control(pDX, IDC_ALREADY_AFFECTED, m_AlreadyAffected);
	DDX_Control(pDX, IDC_EDIT_KEY, m_Key);
	DDX_Control(pDX, IDC_CURRENTS, m_Currents);
	DDX_Control(pDX, IDC_COMMANDS, m_Commands);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAccelMapPage, CPropertyPage)
	//{{AFX_MSG_MAP(CAccelMapPage)
	ON_BN_CLICKED(IDC_ASSIGN, OnAssign)
	ON_BN_CLICKED(IDC_REMOVE, OnRemove)
	ON_BN_CLICKED(IDC_RESET, OnReset)
	ON_LBN_SELCHANGE(IDC_COMMANDS, OnSelChangeCommands)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CAccelMapPage message handlers
/////////////////////////////////////////////////////////////////////////////
//
//
BOOL CAccelMapPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	if (m_pAccelManager == NULL) {
		EndDialog(IDCANCEL);
		return FALSE;
	}
	// Remove OK & Cancel, and move the 3 others buttons
	CRect rect;
	GetDlgItem(IDOK)->GetWindowRect(rect);
	int iHeight = rect.Height();
	ScreenToClient(rect);
	GetDlgItem(IDC_ASSIGN)->MoveWindow(rect);
	rect.OffsetRect(0, iHeight);
	GetDlgItem(IDC_REMOVE)->MoveWindow(rect);
	rect.OffsetRect(0, iHeight);
	GetDlgItem(IDC_RESET)->MoveWindow(rect);

	GetDlgItem(IDOK)->ShowWindow(SW_HIDE);
	GetDlgItem(IDCANCEL)->ShowWindow(SW_HIDE);
	
	// Initialize the helper
	m_helper.SetAccelManager(m_pAccelManager);
	m_helper.SetGUIObjects(&m_Key, &m_Currents, &m_Commands, &m_AlreadyAffected);

	// Control the number of commands available.
	if (m_pAccelManager->IsMapStringCommandsEmpty()) {

		((CButton*)GetDlgItem(IDC_ASSIGN))->EnableWindow(FALSE);
		((CButton*)GetDlgItem(IDC_REMOVE))->EnableWindow(FALSE);
		((CButton*)GetDlgItem(IDC_RESET))->EnableWindow(FALSE);

		m_Key.EnableWindow(FALSE);

	} else {
		// Init the commands listbox
		InitCommands();
	}
	// If the programmer doesn't called CreateDefaultTable()
	if (!m_pAccelManager->IsDefaultTableAvailable())
		((CButton*)GetDlgItem(IDC_RESET))->EnableWindow(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


/////////////////////////////////////////////////////////////////////////////
//
//
void CAccelMapPage::OnSelChangeCommands() 
{
	m_helper.SelChangeCommands();
	UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
//
//
void CAccelMapPage::OnAssign() 
{
	UpdateData();
	m_helper.Assign();
	UpdateData(FALSE);
}



/////////////////////////////////////////////////////////////////////////////
//
//
void CAccelMapPage::OnRemove() 
{
	m_helper.Remove();
	UpdateData(FALSE);
}


/////////////////////////////////////////////////////////////////////////////
//
//
void CAccelMapPage::OnReset() 
{
	m_helper.Reset();
}


/////////////////////////////////////////////////////////////////////////////
//
//
void CAccelMapPage::InitCommands()
{
	m_helper.InitCommands();
}

